#include <stdlib.h>			// general purpose standard library
#include <stdio.h>			// standard input/output header
#include <string.h>
#include "host.h"
#include "iso14443a.h"
#include "iso14443b.h"
#include "iso15693.h"
#include "felica.h"
#include "mcu.h"
#include "trf797x.h"
#include "types.h"
#include "uart.h"

//===============================================================

u08_t buf[100];

//u08_t enable = 0;

u08_t i_reg = 0x01;							// interrupt register

u08_t irq_flag = 0x00;
u08_t rx_error_flag = 0x00;
s08_t rxtx_state = 1;							// used for transmit recieve byte count

s16_t nfc_state;

u08_t remote_flag = 0;
u08_t stand_alone_flag = 1;
u08_t reader_mode = 0x00;					// determines how interrups will be handled



//===============================================================
//
//===============================================================

void main (void)	{
	
	// Settings
	WDTCTL = WDTPW + WDTHOLD;			// stop watchdog timer
	
	McuDelayMillisecond(2);				// wait until system clock started

	SLAVE_SELECT_PORT_SET;							// P3.0 - Slave Select
	SLAVE_SELECT_HIGH;

	#ifdef ENABLE_HOST
		UartSetup();					// settings for UART
	#endif

	McuDelayMillisecond(3);				// wait until system clock started

	ENABLE_SET;							// P1.0 is switched in output direction
	TRF_ENABLE;

	Trf797xCommunicationSetup();		// settings for communication with TRF

	McuDelayMillisecond(1);				// wait until system clock started
		
	Trf797xInitialSettings();			// Set MCU Clock Frequency to 6.78 MHz and OOK Modulation
	
	// Now switch from DCO to external 6.78 MHz clock

	McuOscSel(0x00);       		// set the oscillator
	McuDelayMillisecond(10);
	

	// Re-configure the USART with this external clock

	Trf797xReConfig();
     	
	ENABLE_INTERRUPTS;					// General enable interrupts

	LED_ALL_OFF;
	LED_POWER_ON;						// board ist powered, green LED
    
	OOK_DIR_IN;							// set OOK port tristate
//	enable = 1;							// indicates, that setting are done

	reader_mode = 0x00;
	stand_alone_flag = 1;				// stand alone mode
	remote_flag = 0;					// host is not active
    // -------------------	Settings done ----------------------------------------------------------------

	HostProtocolSet();
	Msp430f23x0Counter2Set();


	while(1)
	{
		HostInventoryGet();
		McuDelayMillisecond(200);

		if(isSeries == 1 && IsSeriesDelay() ==1 )
		{
			isSeries = 0;
			// wake up cc3200
			LED_14443A_ON;
			McuDelayMillisecond(200);
			LED_14443A_OFF;
			// read cc3200 command
			if(UartGetFrame(buf) == 1)
			{
				if(strncmp((char*)buf,"^Get?",4) == 0)
				{
					sprintf((char*)buf,"#%s,%d\r\n",uid,nfcCounter);
					UartSendCString((char*)buf);
					nfcCounter = 0;
				}
				else
					UartSendCString("nfc<->cc3200 communication error\r\n");
			}
			else
				UartSendCString("nfc<->cc3200 connection error\r\n");
		}
	}

}
